@@ -47,6 +47,7 @@
 #include "oobe.h"
 #include "bootman.h"
 #include "swapman.h"
+#include "throbber.h"
 
 #include "minstall.h"
 
@@ -276,26 +277,6 @@ void MInstall::startup()
     gotoPage(Step::TERMS);
 }
 
-void MInstall::splashSetThrobber(bool active) noexcept
-{
-    if (active) {
-        if (throbber) return;
-        gui.labelSplash->installEventFilter(this);
-        throbber = new QTimer(this);
-        connect(throbber, &QTimer::timeout, this, [this]() noexcept {
-            ++throbPos;
-            gui.labelSplash->update();
-        });
-        throbber->start(120);
-    } else {
-        if (!throbber) return;
-        delete throbber;
-        throbber = nullptr;
-        gui.labelSplash->removeEventFilter(this);
-    }
-    gui.labelSplash->update();
-}
-
 // turn auto-mount off and on
 void MInstall::setupAutoMount(bool enabled)
 {
@@ -567,11 +548,16 @@ int MInstall::showPage(int curr, int next) noexcept
     if (next == Step::SPLASH) { // Enter splash screen
         gui.boxMain->setCursor(Qt::WaitCursor);
         appConf.setSection(u"GUI"_s);
-        splashSetThrobber(appConf.getBoolean(u"SplashThrobber"_s, true));
+        if (appConf.getBoolean(u"SplashThrobber"_s, true)) {
+            throbber = new Throbber(gui.labelSplash);
+        }
         appConf.setSection();
     } else if (curr == Step::SPLASH) { // Leave splash screen
         gui.labelSplash->clear();
-        splashSetThrobber(false);
+        if (throbber) {
+            delete throbber;
+            throbber = nullptr;
+        }
         gui.boxMain->unsetCursor();
     } else if (curr == Step::TERMS && next > curr) {
         if (modeOOBE) return Step::NETWORK;
@@ -588,14 +574,18 @@ int MInstall::showPage(int curr, int next) noexcept
             return Step::PARTITIONS;
         }
     } else if (curr == Step::REPLACE && next > curr) {
+        gui.boxMain->setCursor(Qt::WaitCursor);
         if (!replacer->validate()) {
             nextFocus = gui.tableExistInst;
+            gui.boxMain->unsetCursor();
             return curr;
         }
         if (!replacer->preparePartMan()) {
+            gui.boxMain->unsetCursor();
             return curr;
         }
         if (!pretend && !(base && base->saveHomeBasic())) {
+            gui.boxMain->unsetCursor();
             QMessageBox msgbox(this);
             msgbox.setIcon(QMessageBox::Critical);
             msgbox.setText(tr("The data in /home cannot be preserved because"
@@ -603,9 +593,12 @@ int MInstall::showPage(int curr, int next) noexcept
             msgbox.exec();
             return curr;
         }
+        gui.boxMain->unsetCursor();
         return Step::CONFIRM;
     } else if (curr == Step::REPLACE && next < curr) {
+        gui.boxMain->setCursor(Qt::WaitCursor);
         replacer->clean();
+        gui.boxMain->unsetCursor();
     } else if (curr == Step::PARTITIONS) {
         if (next > curr) {
             if (!partman->validate(automatic)) {
@@ -642,10 +635,10 @@ int MInstall::showPage(int curr, int next) noexcept
                     abortEndUI(false);
                     return Step::SPLASH;
                 }
-                next = (advanced ? Step::BOOT : Step::SWAP);
             } else if (!gui.radioReplace->isChecked()) {
                 advanced = true;
             }
+            next = (advanced ? Step::BOOT : Step::SWAP);
             bootman->buildBootLists();
             swapman->setupDefaults();
             loadConfig(2);
@@ -733,6 +726,9 @@ void MInstall::pageDisplayed(int next) noexcept
     case Step::INSTALLATION:
         gui.textHelp->setText("<p><b>"_L1 + tr("Installation Options") + "</b><br/>"_L1
             + tr("If you are running Mac OS or Windows OS (from Vista onwards), you may have to use that system's software to set up partitions and boot manager before installing.") + "</p>"_L1
+            "<p><b>"_L1 + tr("Dual drive") + "</b><br/>"_L1
+            + tr("If your system has multiple storage drives, this option allows you to have the system files on one drive (the System drive),"
+                " while keeping the data of all users on a separate drive (the Home drive).") + "</p>"_L1
             "<p><b>"_L1 + tr("Using the root-home space slider") + "</b><br/>"_L1
             + tr("The drive can be divided into separate system (root) and user data (home) partitions using the slider.") + "</p>"_L1
             "<p>"_L1 + tr("The <b>root</b> partition will contain the operating system and applications.") + "<br/>"_L1
@@ -749,15 +745,19 @@ void MInstall::pageDisplayed(int next) noexcept
             + tr("If you need more control over where %1 is installed to, select \"<b>%2</b>\" and click <b>Next</b>."
                 " On the next page, you will then be able to select and configure the storage devices and"
                 " partitions you need.").arg(PROJECTNAME, gui.radioCustomPart->text().remove('&')) + "</p>"_L1
-                              + "<p>" + "<b>"_L1 + tr("Replace existing installation") + "</b><br/>"_L1 + tr("Replace existing installation option will attempt to replace an existing installation with the same "
-                                           "disk configuration as the existing installation.  Home directories are preserved."));
+            "<p><b>"_L1 + tr("Replace existing installation") + "</b><br/>"_L1 + tr("Replace existing installation option will attempt to replace an existing installation with the same "
+                "disk configuration as the existing installation.  Home directories are preserved."));
         break;
 
     case Step::REPLACE:
         gui.textHelp->setText("<p><b>"_L1 + tr("Replace existing installation") + "</b><br/>"_L1
             + tr("If you have an existing installation, you can use this function to replace it with a fresh installation.") + "</p>"_L1
-            "<p>"_L1 + tr("This is particularly useful if you are upgrading from a previous version and want to preserve your data.") + "<br/>"_L1
-            + tr("There is no guarantee of this working successfully. Ensure you have a good working backup of all important data before continuing.") + "</p>"_L1);
+            "<p>"_L1 + tr("This is particularly useful if you are upgrading from a previous version and want to preserve your data.") + "</p>"_L1
+            "<p><b>"_L1 + tr("Warning") + "</b><br/>"_L1
+            + tr("There is no guarantee of this working successfully. Ensure you have a good working backup of all important data before continuing.") + "</p>"_L1
+            "<p>"_L1 + tr("This feature is designed to replace an installation performed using the regular install method,"
+                " and may fail to replace an installation with a complex layout or storage scheme. Corruption or data loss may occur.") + "<br/>"_L1
+            + tr("To replace an installation with a complex layout or storage scheme, it is recommended to use the custom layout option instead.") + "</p>"_L1);
 
         if(gui.tableExistInst->rowCount() <= 0) {
             QTimer::singleShot(0, gui.pushReplaceScan, &QPushButton::click);
@@ -1074,36 +1074,7 @@ void MInstall::gotoPage(int next) noexcept
 
 bool MInstall::eventFilter(QObject *watched, QEvent *event) noexcept
 {
-    if (event->type() != QEvent::Paint) {
-        return QDialog::eventFilter(watched, event);
-    } else if (watched == gui.labelSplash) {
-        // Setup needed to draw the load indicator.
-        QPainter painter(gui.labelSplash);
-        painter.setRenderHints(QPainter::Antialiasing);
-        const int lW = gui.labelSplash->width(), lH = gui.labelSplash->height();
-        painter.translate(lW / 2, lH / 2);
-        painter.scale(lW / 200.0, lH / 200.0);
-        // Draw the load indicator on the splash screen.
-        static constexpr int blades = 12;
-        static constexpr qreal angle = 360.0 / blades;
-        painter.rotate(angle * throbPos);
-        float hue = 1.0, alpha = 0.18;
-        const float huestep = std::min(throbPos, 60) / (60.0 * blades);
-        static constexpr float alphastep = 0.18 / blades;
-        QPen pen;
-        pen.setWidth(3);
-        pen.setJoinStyle(Qt::MiterJoin);
-        for (unsigned int ixi=blades; ixi>0; --ixi) {
-            const QColor &color = QColor::fromHsvF(hue, 1.0, 1.0, alpha);
-            hue -= huestep, alpha += alphastep;
-            painter.setBrush(color);
-            pen.setColor(color.darker());
-            painter.setPen(pen);
-            static constexpr QPoint blade[] = {{-15, -6}, {15, -75}, {0, -93}, {-15, -75}};
-            painter.drawConvexPolygon(blade, 4);
-            painter.rotate(angle);
-        }
-    } else if (watched == gui.textHelp) {
+    if (watched == gui.textHelp && event->type() == QEvent::Paint) {
         // Draw the help pane backdrop image, which goes through the alpha-channel background.
         QPainter painter(gui.textHelp);
         painter.setRenderHints(QPainter::Antialiasing);
@@ -1137,7 +1108,9 @@ void MInstall::closeEvent(QCloseEvent *event) noexcept
         gui.labelSplash->clear();
         gotoPage(Step::SPLASH);
         proc.unhalt();
-        proc.exec(u"/usr/sbin/shutdown"_s, {u"-hP"_s, u"now"_s});
+        if (!pretend) {
+            proc.exec(u"/usr/sbin/shutdown"_s, {u"-hP"_s, u"now"_s});
+        }
     } else {
         // Fully aborted installation (but not OOBE).
         event->accept();
@@ -1179,7 +1152,10 @@ void MInstall::abortEndUI(bool closenow) noexcept
 {
     if (closenow) qApp->exit(EXIT_FAILURE); // this->close() doesn't work.
     else {
-        splashSetThrobber(false);
+        if (throbber) {
+            delete throbber;
+            throbber = nullptr;
+        }
         gui.boxMain->unsetCursor();
         // Close should be the right button at this stage.
         disconnect(gui.pushNext);
